import 'dart:async';
import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:f_k_chart/common/bloc_provider.dart';
// import 'package:f_k_chart/constants/main.dart';
import 'package:f_k_chart/flutter_k_chart/chart_style.dart';
import 'package:f_k_chart/flutter_k_chart/entity/info_window_entity.dart';
import 'package:f_k_chart/flutter_k_chart/entity/k_line_entity.dart';
import 'package:f_k_chart/flutter_k_chart/k_chart_widget.dart';
import 'package:f_k_chart/flutter_k_chart/kline_scroll_bloc.dart';
import 'package:f_k_chart/flutter_k_chart/renderer/chart_painter.dart';
import 'package:f_k_chart/flutter_k_chart/renderer/main_chart_painter.dart';
import 'package:f_k_chart/flutter_k_chart/renderer/secondary_chart_painter.dart';
import 'package:f_k_chart/flutter_k_chart/utils/date_format_util.dart';
import 'package:f_k_chart/flutter_k_chart/utils/number_util.dart';
// import 'package:f_k_chart/utils/common_util.dart';
import 'package:f_k_chart/flutter_k_chart/dashed_rect.dart';

// ignore: must_be_immutable
class KlineScrollWidget extends StatefulWidget {
  KlineScrollWidget(
    this.datas, {
    Key key,
    this.isLine = true,
    this.mainState = MainState.MA,
    this.secondaryState = SecondaryState.MACD,
    this.pointWidth = ChartStyle.pointWidth,
    this.canScale = true,
    this.maxRectWidth,
    this.minRectWidth,
    this.klineType,
    @required this.uniqueMark,
    this.userFriendlyMinPointWidth = ChartStyle.userFriendlyMinPointWidth,
  }) : super(key: key);

  final List<KLineEntity> datas;
  final MainState mainState;
  final SecondaryState secondaryState;
  final bool isLine;
  final double pointWidth;
  final bool canScale;
  final double maxRectWidth;
  final double minRectWidth;
  final String klineType;
  final String uniqueMark;
  final double userFriendlyMinPointWidth;

  @override
  State<StatefulWidget> createState() {
    // TODO: implement createState
    return KlineScrollWidgetState();
  }
}

class KlineScrollWidgetState extends State<KlineScrollWidget> {
  KlineScrollBloc bloc;

  ScrollController _controller;

  @override
  void initState() {
    super.initState();
    bloc = KlinePageScrollBloc(
      rectWidth: widget.pointWidth,
      maxRectWidth: widget.maxRectWidth,
      minRectWidth: widget.minRectWidth,
      userFriendlyMinPointWidth: widget.userFriendlyMinPointWidth,
    );
  }

  @override
  void didUpdateWidget(KlineScrollWidget oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.datas.toString() != widget.datas.toString()) {
      bloc.updateDataList(widget.datas);
    }
    if (oldWidget.uniqueMark != widget.uniqueMark) {
      bloc.setMaxRectWidth(widget.maxRectWidth);
      bloc.setMinRectWidth(widget.minRectWidth);
      if(widget.pointWidth > 0){
        bloc.setRectWidth(widget.pointWidth);
      }
      if (_controller != null &&
          bloc.allKlineList != null &&
          _controller.hasClients) {
        _controller.jumpTo(
            bloc.rectWidth * bloc.allKlineList.length - bloc.screenWidth);
      }
    }
  }

  @override
  void dispose() {
    bloc?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    // TODO: implement build
    double lastScale;
    double currentRectWidth;
    bool isScale = false;

    bool isLongPress = false;
    double mSelectX = 0.0;

    _controller = ScrollController(
        initialScrollOffset:
            bloc.rectWidth * bloc.allKlineList.length - bloc.screenWidth);
    _controller.addListener(() {
      int visiblePointCount =
          KlineScrollBloc.calculateVisiblePointCountFitScreenWidth(
              bloc.screenWidth, bloc.rectWidth);
      int startVisibleIndex = (_controller.offset ~/ bloc.rectWidth).toInt();

      if (bloc.allKlineList == null || visiblePointCount == null) {
        return;
      }
      bloc.calculateVisibleKlineList(
          startVisibleIndex, visiblePointCount, bloc.allKlineList.length);
    });

    return BlocProvider<KlineScrollBloc>(
        //key: PageStorageKey('market'),
        bloc: bloc,
        child: GestureDetector(
            onScaleStart: (details) {
              if (widget.canScale) {
                currentRectWidth = bloc.rectWidth;
                isScale = true;
              }
            },
            onScaleUpdate: (details) {
              if (widget.canScale) {
                double scale = details.scale;
                if (scale == 1.0) {
                  return;
                }
                lastScale = details.scale;
                double rectWidth = scale * currentRectWidth;
                bloc.setRectWidth(rectWidth);
                int visiblePointCount =
                    KlineScrollBloc.calculateVisiblePointCountFitScreenWidth(
                        bloc.screenWidth, bloc.rectWidth);
                bloc.calculateVisibleKlineList(bloc.currentIndex,
                    visiblePointCount, bloc.allKlineList.length);
              }
            },
            onScaleEnd: (details) {
              if (widget.canScale) {
                isScale = false;
              }
            },
            onLongPressStart: (details) {
              isLongPress = true;
              if (mSelectX != details.globalPosition.dx) {
                mSelectX = details.globalPosition.dx;
              }
              bloc.setKlineWidgetStatus(
                  isLongPress: isLongPress, selectX: mSelectX);
            },
            onLongPressMoveUpdate: (details) {
              if (mSelectX != details.globalPosition.dx) {
                mSelectX = details.globalPosition.dx;
              }
              bloc.setKlineWidgetStatus(selectX: mSelectX);
            },
            onLongPressEnd: (details) {
              isLongPress = false;
              bloc.setKlineWidgetStatus(isLongPress: isLongPress);
            },
            child: StreamBuilder(
                stream: bloc.outklineList,
                builder: (BuildContext context,
                    AsyncSnapshot<List<KLineEntity>> snapshot) {
                  List<KLineEntity> data = snapshot.data;
                  if (data != null) {
                    double width = MediaQuery.of(context).size.width;
                    bloc.setScreenWidth(width);
                    bloc.setRectWidth(bloc.rectWidth);
                  }
                  return Container(
                    child: Stack(
                      alignment: Alignment.center,
                      fit: StackFit.expand,
                      children: <Widget>[
                        Container(child: Builder(
                          builder: (BuildContext context) {
                            KlineScrollBloc klineScrollBloc =
                                BlocProvider.of<KlineScrollBloc>(context);
                            return StreamBuilder(
                                stream: klineScrollBloc.outKlineWidgetStatus,
                                builder: (BuildContext context,
                                    AsyncSnapshot<KlineWidgetStatus> snapshot) {
                                  KlineWidgetStatus widgetStatus =
                                      KlineWidgetStatus();
                                  if (snapshot.data != null) {
                                    widgetStatus = snapshot.data;
                                  }
                                  return KlineWidgetView(
                                    isLongPress:
                                        widgetStatus.isLongPress ?? false,
                                    mSelectX: widgetStatus.selectX ?? 0.0,
                                    mPointWidth: widgetStatus.pointWidth ??
                                        ChartStyle.pointWidth,
                                    isLine: widget.isLine,
                                    mainState: widget.mainState,
                                    secondaryState: widget.secondaryState,
                                  );
                                });
                          },
                        )),
                        if (data != null)
                          Scrollbar(
                              child: SingleChildScrollView(
                            child: Container(
                              width: (bloc.rectWidth * data.length ?? 0)
                                  .toDouble(),
                            ),
                            controller: _controller,
                            scrollDirection: Axis.horizontal,
                          )),
                      ],
                    ),
                  );
                })));
  }
}

class KlinePageScrollBloc extends KlineScrollBloc {
  @override
  //重写init方法
  void initData() {
    this.updateDataList([]);
    super.initData();
  }

  KlinePageScrollBloc(
      {rectWidth = ChartStyle.pointWidth,
      maxRectWidth,
      minRectWidth,
      userFriendlyMinPointWidth = ChartStyle.userFriendlyMinPointWidth}) {
    super.rectWidth = rectWidth;
    super.minRectWidth = minRectWidth;
    super.maxRectWidth = maxRectWidth;
    super.userFriendlyMinPointWidth = userFriendlyMinPointWidth;
  }
}

// ignore: must_be_immutable
class KlineWidgetView extends StatelessWidget {
  bool isLongPress;
  double mSelectX;
  double mPointWidth;

  final MainState mainState;
  final SecondaryState secondaryState;
  final bool isLine;

  KlineWidgetView({
    Key key,
    this.isLongPress = false,
    this.mSelectX = 0.0,
    this.mPointWidth = ChartStyle.pointWidth,
    this.isLine = true,
    this.mainState = MainState.MA,
    this.secondaryState = SecondaryState.MACD,
  });

  @override
  Widget build(BuildContext context) {
    KlineScrollBloc klineScrollBloc = BlocProvider.of<KlineScrollBloc>(context);
    return StreamBuilder(
        stream: klineScrollBloc.outCurrentKlineList,
        builder:
            (BuildContext context, AsyncSnapshot<List<KLineEntity>> snapshot) {
          List<KLineEntity> dataList = snapshot.data ?? [];
          return Column(
            children: <Widget>[
              Expanded(
                child: KScrollChartWidget(dataList,
                    isLine: isLine,
                    mainState: mainState,
                    secondaryState: secondaryState,
                    isLongPress: isLongPress,
                    mSelectX: mSelectX,
                    mPointWidth: mPointWidth,
                    mStartIndexInWholeList: klineScrollBloc.getCurrentIndex(),
                    mGap: klineScrollBloc.getGap()),
              )
            ],
          );
        });
  }
}

class KScrollChartWidget extends StatefulWidget {
  List<KLineEntity> datas;
  MainState mainState;
  SecondaryState secondaryState;
  bool isLine;
  bool isLongPress;
  double mSelectX;
  double mPointWidth;
  int mStartIndexInWholeList;
  int mGap;

  KScrollChartWidget(
    this.datas, {
    this.mainState = MainState.MA,
    this.secondaryState = SecondaryState.MACD,
    this.isLine,
    this.isLongPress = false,
    this.mSelectX = 0.0,
    this.mPointWidth = ChartStyle.pointWidth,
    this.mStartIndexInWholeList = 0,
    this.mGap = 1,
  });

  @override
  _KScrollChartWidgetState createState() => _KScrollChartWidgetState();
}

class _KScrollChartWidgetState extends State<KScrollChartWidget> {
  double mScaleX = 1.0, mScrollX = 0.0;
  StreamController<InfoWindowEntity> mInfoWindowStream;
  double mWidth = 0;

  @override
  void initState() {
    super.initState();
    mInfoWindowStream = StreamController<InfoWindowEntity>();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    mWidth = MediaQuery.of(context).size.width;
  }

  @override
  void didUpdateWidget(KScrollChartWidget oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.datas != widget.datas) {
      mScrollX = 0.0;
      mScaleX = 1.0;
    }
  }

  @override
  void dispose() {
    mInfoWindowStream?.close();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (widget.datas == null || widget.datas.isEmpty) {
      mScrollX = 0.0;
      mScaleX = 1.0;
    }
    final Widget dash = Container(
      width: getWidth(),
      child: DashedRect(color: ChartColors.hintGrey, strokeWidth: 0.5, gap: 3.0),
    );
    return Stack(
      children: <Widget>[
//        CustomPaint(
//          size: Size(double.infinity, double.infinity),
//          painter: ChartPainter(
//            datas: widget.datas,
//            scaleX: mScaleX,
//            scrollX: mScrollX,
//            selectX: widget.mSelectX,
//            isLongPass: widget.isLongPress,
//            mainState: widget.mainState,
//            secondaryState: widget.secondaryState,
//            isLine: widget.isLine,
//            sink: mInfoWindowStream?.sink,
//            mPointWidth: widget.mPointWidth,
//          ),
//        ),
        Column(
          children: <Widget>[
            dash,
            Expanded(
              child: CustomPaint(
                size: Size(double.infinity, double.infinity),
                painter: MainChartPainter(
                  datas: widget.datas,
                  scaleX: mScaleX,
                  scrollX: mScrollX,
                  selectX: widget.mSelectX,
                  isLongPass: widget.isLongPress,
                  mainState: widget.mainState,
                  isLine: widget.isLine,
                  sink: mInfoWindowStream?.sink,
                  mPointWidth: widget.mPointWidth,
                  mStartIndexInWholeList: widget.mStartIndexInWholeList,
                  mGap: widget.mGap,
                ),
              ),
            ),
            Offstage(
              offstage:
                  widget.isLine || widget.secondaryState == SecondaryState.NONE,
              child: SizedBox(
                height: 34.72,
              ),
            ),
            Offstage(
              offstage:
                  widget.isLine || widget.secondaryState == SecondaryState.NONE,
              child: dash,
            ),
            Offstage(
              offstage:
                  widget.isLine || widget.secondaryState == SecondaryState.NONE,
              child: CustomPaint(
                size: Size(double.infinity,getHeight() * 0.15),
                painter: SecondaryChartPainter(
                  datas: widget.datas,
                  scaleX: mScaleX,
                  scrollX: mScrollX,
                  selectX: widget.mSelectX,
                  isLongPass: widget.isLongPress,
                  secondaryState: widget.secondaryState,
                  isLine: widget.isLine,
                  sink: mInfoWindowStream?.sink,
                  mPointWidth: widget.mPointWidth,
                  mStartIndexInWholeList: widget.mStartIndexInWholeList,
                  mGap: widget.mGap,
                ),
              ),
            )
          ],
        ),
        _buildInfoDialog()
      ],
    );
  }

  Widget _buildInfoDialog() {
    List<String> infoNames = ["时间", "开", "高", "低", "收", "涨跌额", "涨幅"];
    List infos;
    return StreamBuilder<InfoWindowEntity>(
        stream: mInfoWindowStream?.stream,
        builder: (context, snapshot) {
          if (!widget.isLongPress ||
              widget.isLine == true ||
              !snapshot.hasData ||
              snapshot.data.kLineEntity == null) return Container();
          KLineEntity entity = snapshot.data.kLineEntity;
          double upDown = entity.close - entity.open;
          double upDownPercent = upDown / entity.open * 100;
          infos = [
            getDate(entity.time),
            entity.open,
            entity.high,
            entity.low,
            entity.close,
            "${upDown > 0 ? "+" : ""}${NumberUtil.formatUpDownValue(upDown)}",
            "${upDownPercent > 0 ? "+" : ''}${upDownPercent.toStringAsFixed(2)}%",
            NumberUtil.format(entity.vol)
          ];
          return Container(
            margin: EdgeInsets.only(
                left: snapshot.data.isLeft ? 4 : mWidth - mWidth / 3 - 4,
                top: 25),
            width: mWidth / 3,
            decoration: BoxDecoration(
                color: ChartColors.selectFillColor,
                border: Border.all(
                    color: ChartColors.selectBorderColor, width: 0.5)),
            child: ListView.builder(
              padding: EdgeInsets.all(4),
              itemCount: infoNames.length,
              itemExtent: 14.0,
              shrinkWrap: true,
              itemBuilder: (context, index) {
                return _buildItem(infos[index].toString(), infoNames[index]);
              },
            ),
          );
        });
  }

  Widget _buildItem(String info, String infoName) {
    Color color = Colors.white;
    if (info.startsWith("+"))
      color = Colors.green;
    else if (info.startsWith("-"))
      color = Colors.red;
    else
      color = Colors.white;
    return Row(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Expanded(
            child: Text("$infoName",
                style: const TextStyle(color: Colors.white, fontSize: 10.0))),
        Text(info, style: TextStyle(color: color, fontSize: 10.0)),
      ],
    );
  }

  String getDate(int date) {
    return dateFormat(DateTime.fromMillisecondsSinceEpoch(date),
        [yy, '-', mm, '-', dd, ' ', HH, ':', nn]);
  }

  double getHeight() {
    return window.physicalSize.height / window.devicePixelRatio;
  }

  double getWidth() {
    return window.physicalSize.width / window.devicePixelRatio;
  }
}
